﻿using System;
using System.ServiceModel;
using System.Threading.Tasks;
using System.Windows;
using Contract;

namespace WpfApplication1
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private IWorker worker;

        public MainWindow()
        {
            //start the supervisor service
            ServiceHost host = new ServiceHost(typeof(Supervisor));
            host.AddServiceEndpoint(typeof(ISupervisor), new NetNamedPipeBinding(), "net.pipe://localhost/Supervisor");
            host.Open();

            //create the channel to the worker
            EndpointAddress endpointAddress = new EndpointAddress("net.pipe://localhost/Worker");
            NetNamedPipeBinding binding = new NetNamedPipeBinding();
            worker = ChannelFactory<IWorker>.CreateChannel(binding, endpointAddress);

            //register the call back
            Supervisor.action = (progress) =>
            {
                Dispatcher.BeginInvoke(new Action(() =>
                {
                    progressBar.Value = progress;
                }));
            };

            InitializeComponent();
        }

        private void RunButton_Click(object sender, RoutedEventArgs e)
        {
            Task.Factory.StartNew(() =>
            {
                worker.RunTask(30);
            });
        }

        private void PrintButton_Click(object sender, RoutedEventArgs e)
        {
            worker.Print();
        }
    }

    [ServiceBehavior(InstanceContextMode = InstanceContextMode.Single, ConcurrencyMode = ConcurrencyMode.Reentrant)]
    public class Supervisor : ISupervisor
    {
        public static Action<int> action;

        public void Report(int progress)
        {
            action(progress);
        }
    }
}
